unit TouchScr;

(*********************************************
TTouchScroller -> TImage

Assign the ScrollBox property to a TScrollBox on your form.
Assign a picture property to the control that represents
visually the control(s) within the TScrollBox.  Now,
when you click and drag inside the control, the scroll
bars in the TScrollBox will move accordingly, reflecting the
position of your mouse in this control!

PROPERTIES:

ScrollBox - Hook this into a TScrollBox on the form.
*********************************************)

interface

uses
  WinTypes, WinProcs, SysUtils, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TTouchScroller = class( TImage )
  private
     FScrollBox: TScrollBox;
     bMoving: boolean;
     FMouseX, FMouseY: integer;
  protected
     xRatioX, xRatioY: single;
     procedure MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: integer ); override;
     procedure MouseMove( Shift: TShiftState; X, Y: integer ); override;
     procedure MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: integer ); override;
     procedure SetScrollBox( sb: TScrollBox );
     procedure SetRange;
  public
     constructor Create( AOwner: TComponent ); override;
  published
     property ScrollBox: TScrollBox read FScrollBox write SetScrollBox;
     property MouseX: integer read FMouseX;
     property MouseY: integer read FMouseY;
  end;

procedure Register;

implementation

constructor TTouchScroller.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  AutoSize := FALSE;
  Stretch := TRUE;
  xRatioX := 0.0;
end;

procedure TTouchScroller.SetScrollBox( sb: TScrollBox );
begin
  if FScrollBox <> sb then
     begin
        xRatioX := 0.0;
        FScrollBox := sb;
        SetRange;
     end;
end;

procedure TTouchScroller.SetRange;
begin
  if xRatioX = 0.0 then
     if FScrollBox <> nil then
        if FScrollBox.HorzScrollBar.Range > 0 then
           begin
              xRatioX := ( FScrollBox.HorzScrollBar.Range + FScrollBox.Width ) / Width;
              xRatioY := ( FScrollBox.VertScrollBar.Range + FScrollBox.Height ) / Height;
           end;
end;

procedure TTouchScroller.MouseDown( Button: TMouseButton; Shift: TShiftState; X, Y: integer );
begin
  bMoving := TRUE;
  inherited MouseDown( Button, Shift, X, Y );
  SetRange;
end;

procedure TTouchScroller.MouseMove( Shift: TShiftState; X, Y: integer );
begin
  if bMoving and ( FScrollBox <> nil ) then
     begin
        SetRange;
        if xRatioX = 0 then
           SetRange;

        FMouseX := Trunc( xRatioX * X ) - FScrollBox.Width;
        FMouseY := Trunc( xRatioY * Y ) - FScrollBox.Height;

        if ( FMouseX >= 0 ) and ( FMouseX <= FScrollBox.HorzScrollBar.Range ) then
           FScrollBox.HorzScrollBar.Position := FMouseX
        else if FMouseX < 0 then
           FScrollBox.HorzScrollBar.Position := 0
        else if FMouseX > FScrollBox.HorzScrollBar.Range then
           FScrollBox.HorzScrollBar.Position := FScrollBox.HorzScrollBar.Range;

        if ( FMouseY >= 0 ) and ( FMouseY <= FScrollBox.VertScrollBar.Range ) then
           FScrollBox.VertScrollBar.Position := FMouseY
        else if FMouseY < 0 then
           FScrollBox.VertScrollBar.Position := 0
        else if FMouseY > FScrollBox.VertScrollBar.Range then
           FScrollBox.VertScrollBar.Position := FScrollBox.VertScrollBar.Range;

     end;
  inherited MouseMove( Shift, X, Y );
end;

procedure TTouchScroller.MouseUp( Button: TMouseButton; Shift: TShiftState; X, Y: integer );
begin
  bMoving := FALSE;
  inherited MouseUp( Button, Shift, X, Y );
end;

procedure Register;
begin
  RegisterComponents( 'Vis', [TTouchScroller] );
end;

end.
